###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

#from AccessControl import allow_class
#from Interface.Implements import implements
from IObjects import ISql
from Settings import registerRakunClass


class BaseCondition:    
    
    operator = ''
    
    def __init__(self, field, value):
        self.field = field
        self.value = value
        
    def __and__(self, other):
        return And(self,other)
    
    def __or__(self, other):
        return Or(self,other)
    
    def __invert__(self):
        return Not(self)
    
    def __eq__(self, other):
        return isinstance(other, BaseCondition) and (self.field == other.field)\
               and (self.operator == other.operator) and (self.value == other.value)
        
        
    def render(self, dbobject):
        if ISql.isImplementedBy(self.value):
            value = self.value.render_sql_condition()
        else:
            value = dbobject.setEscapeChars(self.value)
            value = self.field.render_sql_value(dbobject, value)
            
        return '(%s %s %s)' %(self.field.render_sql_condition(),
                              self.operator, 
                              value )


class Eq(BaseCondition):    
    
    operator = '='

registerRakunClass('Eq', Eq)

class Ge(BaseCondition):
    
    operator = '>='

registerRakunClass('Ge', Ge)

class Gt(BaseCondition):    
    
    operator = '>'

registerRakunClass('Gt',Gt)    


class Le(BaseCondition):    

    operator = '<='

registerRakunClass('Le', Le)


class Lt(BaseCondition):
    
    operator = '<'

registerRakunClass('Lt', Lt)


class Ne(BaseCondition):

    operator = '<>'
    
registerRakunClass('Ne', Ne)


class MatchGlobe(BaseCondition):    
    
    def render(self, dbobject):
        dbtype = self.field.get_db_type()
        if dbtype in ['string','text', 'date']:
            value = '%' + self.value + '%'
            # (fieldname like value)
            return "(%s %s '%s')" %(self.field.render_sql_condition(), 
                                       dbobject.like_keyword,
                                       dbobject.setEscapeChars(value) )
        else:
            newCondition = Eq(self.field, self.value)
            return newCondition.render(dbobject)

registerRakunClass('MatchGlobe', MatchGlobe)


class In(BaseCondition):

    def render(self, dbobject=None):
        """        
        initial value type must be tuple 
        example : In(field,(1,2,3))
        Note: subquery isn't support
        """        
        return "(%s IN %s)"%(self.field.render_sql_condition(), str(self.value))

registerRakunClass('In', In)


class Between(BaseCondition):
    
    def __init__(self, field, min, max):
        """
        field : Rakun Field object
        min   : sub limit
        max   : upper limit
        """
        self.field = field
        self.min = min
        self.max = max

    def render(self, dbobject=None):
        """
        usage : Between(Field, 5, 10)
        result: Field BETWEEN 5 AND 10
        """
        return "(%s BETWEEN %s AND %s)" %(self.field.render_sql_condition(),
                                        self.min, 
                                        self.max)
    
    def invertRender(self, dbobject=None):
        """
        usage : Not(Between(Field, 5, 10))
        usage : ~Between(Field, 5, 10)  
        result: Field NOT Between 5 AND 10
        """
        return "(%s NOT BETWEEN %s AND %s)" %(self.field.render_sql_condition(),
                                              self.min, 
                                              self.max)

registerRakunClass('Between', Between)


class Not(BaseCondition):
    """ ~(query) """
    
    def __init__(self, condition):
        self.condition = condition
    
    def render(self, dbobject):
        if hasattr(self.condition, 'invertRender'):
            return self.condition.invertRender(dbobject)
        else:
            return "(NOT %s)" %(self.condition.render(dbobject))
    
registerRakunClass('Not', Not)


class CompositeCondition(BaseCondition):
    
    operator = ''
    
    def __init__(self, *conditions):
        self._conditions= []
        for c in conditions: self.addCondition(c)

    def __eq__(self,other):
        return isinstance(other, CompositeCondition) and self._conditions == other._conditions
        
        
        
    def addCondition(self, condition):
        assert isinstance(condition, BaseCondition)
        if condition not in self._conditions:
            self._conditions.append(condition)
    
   
    def render(self, dbobject):
        conditions = [condition.render(dbobject) for condition in self._conditions]
        return ' (%s) ' %self.operator.join(conditions)
        

class And(CompositeCondition):
    operator = ' AND '

registerRakunClass('And', And)


class Or(CompositeCondition):
    
    operator = ' OR '

registerRakunClass('Or', Or)