###############################################################################
#
# Copyright (C) 2005-2006 IDA Software Technologies
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
###############################################################################

# -*- coding: utf-8 -*-
from os import path
from StringIO import StringIO
from AccessControl import ClassSecurityInfo
from AccessControl.SecurityManagement import getSecurityManager
from Globals import InitializeClass
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from OFS.PropertyManager import PropertyManager
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.CMFCore.utils import UniqueObject, getToolByName
from Settings import  RelationToolMetaType, RelationToolName,\
     Relation1MetaType, RelationNMetaType, ApplicationToolName
from Permissions import RakunView, ManagePortal
from RakunBaseAppTool import RakunBaseAppTool

class Relation(SimpleItem, PropertyManager):
    """Rakun Relation Object"""   
    manage_options = PropertyManager.manage_options

    _properties = ({'id':'title', 'type': 'string','mode':'w'},
                   {'id':'master_entity','type':'selection','mode':'w',
                    'select_variable':'getAllEntities','label':'Referenced Table'
                    },
                   {'id':'master_fields','type':'multiple selection','mode':'w',
                    'select_variable':'getMasterFields','label':'Displayed Fields'
                    },
                   {'id':'detail_entity','type':'selection','mode':'w',
                    'select_variable':'getAllEntities','label':'Referencing Table'
                    },
                   {'id':'foreign_field','type':'selection','mode':'w',
                    'select_variable':'getDetailFields','label':'Referencing Field'
                    },
                   )
    
    
    def __init__(self,id,title='',master_entity='', master_fields=[],
                 detail_entity='',foreign_field=''):
        self.id = id
        self.title = title
        self.master_entity = master_entity
        self.master_fields = master_fields
        self.detail_entity = detail_entity
        self.foreign_field = foreign_field  
        
    def manage_editProperties(self, REQUEST):
        """ """
        master_entity = REQUEST.get('master_entity')
        detail_entity = REQUEST.get('detail_entity')
        foreign_field = REQUEST.get('foreign_field')
        relations = self.relation_search(master_entity,detail_entity,
                                         foreign_field)
        if relations and (relations[0]['id']!=self.id):            
            message="Relation exists."
            return self.manage_propertiesForm(self,REQUEST,
                                              manage_tabs_message=message)
        else:
            if master_entity != self.master_entity:
                REQUEST.set('master_fields',[])
            if detail_entity != self.detail_entity:
                REQUEST.set('foreign_field','')
            return PropertyManager.manage_editProperties(self, REQUEST)
            
        
    def getAllEntities(self):
        application_tool = getToolByName(self,ApplicationToolName)
        entities = application_tool.getAllEntities()
        return [''] + [entity[0] for entity in entities]
    
    def getEntityFieldIds(self, entity_path):
        application_tool = getToolByName(self,ApplicationToolName)
        try:
            entity = application_tool.getEntityObject(entity_path)
            return entity.get_field_ids()
        except:
            return []
        
    def getDetailFields(self):
        if self.detail_entity:
            return [''] + self.getEntityFieldIds(self.detail_entity)
        else:
            return ['']
        
    def getMasterFields(self):
        if self.master_entity:
            return self.getEntityFieldIds(self.master_entity)
        else:
            return []
    
    def isBroken(self):
        return self.master_entity == '' or self.master_fields == [] or \
               self.detail_entity == '' or self.foreign_field == ''
    
    def getRelationType(self):
        if self.meta_type == Relation1MetaType:
            return '1'
        else:
            return 'N'
            
    def checkAttributes(self,master_entity=None, detail_entity=None,
                        foreign_field=None):        
        if master_entity is not None and (master_entity != self.master_entity):
            return False
        if detail_entity is not None and (detail_entity != self.detail_entity):
            return False        
        if foreign_field is not None and (foreign_field != self.foreign_field):
            return False
        return True
            
    def getAttributes(self):
        return {'master_entity':self.master_entity,
                'detail_entity':self.detail_entity,
                'master_fields':self.master_fields,
                'foreign_field':self.foreign_field,
                'id':self.id,
                'relation':self.getRelationType()
                }
        
    def xmlAttributes(self):        
        return {'master_entity':self.master_entity,
                'detail_entity':self.detail_entity,
                'master_fields':','.join(self.master_fields),
                'foreign_field':self.foreign_field,
                'type':self.getRelationType(),
                'title':self.title}
    
InitializeClass(Relation)

# -------- Relation object -----------------------------------------------------
manage_addRelation1Form = PageTemplateFile('zmi/manage_addRelation1Form',
                                           globals())

def manage_addRelation1(dispatcher,title='',REQUEST=None):
    """Relation add from management screen """
    id = dispatcher.genId()
    relation = Relation1(id,title)    
    try:
        dispatcher.Destination()._setObject(id,relation)
    except AttributeError:
        dispatcher._setObject(id,relation)    
    if REQUEST is not None:
        dispatcher.manage_main(dispatcher,REQUEST,update_menu=1)
    else:
        return dispatcher._getOb(id)
        

class Relation1(Relation):
    
    
    meta_type = Relation1MetaType
    
InitializeClass(Relation1)


# -------- RelationN object-----------------------------------------------------
manage_addRelationNForm = PageTemplateFile('zmi/manage_addRelationNForm',
                                           globals())

def manage_addRelationN(dispatcher,title='',REQUEST=None):
    """Relation add from management screen """
    id = dispatcher.genId()
    relation = RelationN(id,title)
    try:
        dispatcher.Destination()._setObject(id,relation)
    except AttributeError:
        dispatcher._setObject(id,relation)    
    if REQUEST is not None:
        dispatcher.manage_main(dispatcher,REQUEST,update_menu=1)
    else:
        return dispatcher._getOb(id)
    
class RelationN(Relation):
    
    
    meta_type = RelationNMetaType
    
InitializeClass(RelationN)
    

# -------- Rakun Relation Tool -------------------------------------------------
class RelationTool(UniqueObject, Folder, RakunBaseAppTool):
    """Rakun Relation Management Tool """
    
    
    id = RelationToolName
    meta_type = RelationToolMetaType
    toolicon = 'zmi/relation.gif'
    all_meta_types = ({'name':Relation1MetaType,
                       'action':\
                       'manage_addProduct/Rakun/manage_addRelation1Form',
                       'permission':'View management screens'},
                      {'name':RelationNMetaType,
                       'action':\
                       'manage_addProduct/Rakun/manage_addRelationNForm',
                       'permission':'View management screens'},
                      )    
        
    manage_options = Folder.manage_options +\
                   ({'label':'Export',
                     'action':'exportSettings'},
                    )    
    security = ClassSecurityInfo()  
    manage_main = PageTemplateFile('zmi/manage_relations', globals())
    exportSettings = PageTemplateFile('zmi/exportSettings', globals())
    
    security.declareProtected(ManagePortal, 'manage_export')
    def manage_export(self,export_path,REQUEST=None):
        """rakun relations export method"""
        
        from xml.dom import minidom
        document =  minidom.Document()
        relations = document.createElement('relations')
        for r in self.objectValues():
            if not r.isBroken():
                relation = document.createElement('relation')
                attributes = r.xmlAttributes()
                for key, value in attributes.items():
                    relation.setAttribute(key,value)
                relations.appendChild(relation)    
        document.appendChild(relations)        
        fpath = path.join(export_path, 'relations.xml')
        f = file(fpath,'w')
        f.write(document.toprettyxml())
        f.close()                       
        if REQUEST:         
            message = "Relations are exported to %s \n"%(fpath)
            return self.exportSettings(self,REQUEST,
                                       manage_tabs_message=message)
        
    def importFromXML(self,module, import_path):
        """import method"""
        from xml.dom import minidom
        document = minidom.parse(import_path)
        relations = document.getElementsByTagName('relations')[0]        
        for relation in relations.getElementsByTagName('relation'):
            title = relation.getAttribute('title')
            rtype = relation.getAttribute('type')
            master_entity = relation.getAttribute('master_entity')
            master_fields = relation.getAttribute('master_fields')
            detail_entity = relation.getAttribute('detail_entity')
            foreign_field = relation.getAttribute('foreign_field')
            rid = self.addRelation(master_entity.encode('latin1'),
                                   master_fields.encode('latin1').split(','),
                                   detail_entity.encode('latin1'),
                                   foreign_field.encode('latin1'),
                                   rtype.encode('latin1'),
                                   title)
            self.addInstallItem(module, rid)
            
            
        
        

    def genId(self):
        ids = self.objectIds()
        i = 0
        id = 'relation%s'%i
        while id in ids:
            i += 1
            id = 'relation%s' %i
        return id
    
    # used for migrating old style relations
    def addRelation(self,master_entity, master_fields, detail_entity,
                    foreign_field, relation_type, title=''):
        if relation_type == '1':
            addRelation = manage_addRelation1
        elif relation_type == 'N':
            addRelation = manage_addRelationN
        relation = addRelation(self)
        relation.master_entity = master_entity
        relation.detail_entity = detail_entity
        relation.master_fields = master_fields
        relation.foreign_field = foreign_field
        relation.title = title
        return relation.id
        
        
    
    security.declareProtected(RakunView, 'relation_search')
    def relation_search(self,master_entity=None,detail_entity=None,
                        foreign_field=None):
        """ """
        result = []
        for relation in self.objectValues():
            if (not relation.isBroken()) and\
               relation.checkAttributes(master_entity,detail_entity,
                                        foreign_field):
                result.append(relation.getAttributes())
        return result
    
    security.declareProtected(RakunView,'getRelation')
    def getRelation(self,foreign):
        """ """
        url_tool = getToolByName(self, 'portal_url')
        application_tool = getToolByName(self,ApplicationToolName)        
        relativeUrl = url_tool.getRelativeUrl
        if relativeUrl(foreign).startswith(relativeUrl(application_tool)):
            foreign_path = url_tool.getRelativeContentPath(foreign)
            app_path = url_tool.getRelativeContentPath(application_tool)
            detail_path = '/'.join(foreign_path[len(app_path):-1])
            return self.relation_search(detail_entity=detail_path,
                                        foreign_field=foreign.getId())
        
    security.declareProtected(RakunView,'getMasterFields')
    def getMasterFields(self, foreign):
        """ """
        result = []
        application_tool = getToolByName(self,ApplicationToolName)
        relation = self.getRelation(foreign)
        if relation:
            master_entity = application_tool.restrictedTraverse(\
                relation[0]['master_entity'])            
            for mfield in relation[0]['master_fields']:
                result.append(master_entity.get_field(mfield))
        return result    
        
InitializeClass(RelationTool)    